'Programm:   Demo fr den UKW-Fuchsjagd-Sender "Foxy" mit ATtiny45+MICRF102
'            Sendet zufllige 5er-Gruppen von Morsezeichen A...Z,0...9
'            Das Programm beherrscht alle grundlegenden Morsezeichen und bentigt
'            trotz Hochsprache nur ein Fnftel des 4 KB groen Programmspeichers.
'Sprache:    Bascom-AVR-Basic, siehe www.mcselec.com (freie Version hat fr ATtiny45 keine Einschrnkungen)
'Von:        Roland Walter, DL7UNO, www.rowalt.de
'Datum:      06.02.2007 (letzte nderung)
'Literatur:  Roland Walter: AVR-Mikrocontroller-Lehrbuch, siehe www.rowalt.de
'Kommentar:  Pin PB4 wird als Timer1-Ausgang OC1B verwendet, Timer1 erzeugt den
'            Ton selbstndig.
'-------------------------------------------------------------------------------
'Grundeinstelllungen:
$regfile = "ATtiny45.dat"   'Deklarationen fr den ATtiny45 einbinden
$crystal = 1000000          'Taktfrequenz: 1 MHz (Fabrik-Voreinstellung, Interner RC-Oszillator 8MHz mitb Vorteilung 1/8)
'
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'Einstellbare Werte je nach Geschmack:
Const LEN_DIT    = 80          'Lnge eines Punkts, daraus ergibt sich die Morsegeschwindigkeit
Const LEN_DA     = LEN_DIT*3   'Lnge eines Strichs (mu 3*Punkt sein)
Const LEN_PAUSE  = LEN_DIT*6   'Lnge der Pause zwischen den 5er-Gruppen (optimal: 6 Punkt-Lngen)
Const TONE_FREQU = 140         'Tonfrequenz, Wert 140 ergab sich beim Ausprobieren als recht brauchbar
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'Variablen und Konstanten dimensionieren:
Dim i As Byte                  'Fr temporren Gebrauch
Dim Table(36) As Byte          'Feld fr die codierten Morsebuchstaben (wir haben 36 in unserer Tabelle)
Dim CharNo As Byte             'Zhlt die 5 Buchstaben der 5er-Gruppen
Dim CharIndex As Byte          'Index des aktuellen Morsebuchstabens in unserer DATA-Tabelle
Dim BitNo As Byte              'Zhlt die Code-Bits des aktuellen Morsebuchstabens hoch
Dim wEeRndInit As ERAM Word    'Initialisierungswert fr den Pseudo-Zufallsgenerator im EEPROM
Dim ___rseed As Word           'Initialisierungs-Variable des Pseudo-Zufallsgenerators, siehe Bascom-Hilfe unter "Rnd()"
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'Initialisierung des Programms:
'
ACSR.ACD = 0           'Analog-Komparator ausschalten, spart etwas Strom
'
DDRB  = &B00011000     'Pins PB3+PB4 als Ausgnge konfigurieren (1), Rest ist Eingang (0)
PORTB = &B00000111     'PullUp-Widerstnde fr die Pins PB0,PB1,PB2 aktivieren (Strfestigkeit)
'
PORTB.3 = 1            'Pin PB3 auf High (StandBy-Pin am MICRF102): Standby-Modus abschalten
'
'Einlesen der Codes fr die Morsebuchstaben aus DATA-Zeile (siehe unten):
Restore Morsecodes
For i=1 To 36          'Wir haben einen Vorrat von 36 Morsebuchstaben: Alle einlesen
  Read Table(i)
Next i
'
'Zufallsgenerator zur Auswahl der jeweiligen Morsebuchstaben initialisieren:
'Damit der Pseudo-Zufallsgenerator nicht jedesmal nach dem Reset die gleichen Zeichenfolgen erzeugt, mssen wir ihn mit einem
'vernderlichen Startwert fttern. Wir lsen das auf einfachste Weise, indem wir hochzhlen, wie oft das Programm bereits
'gestartet wurde und die Bits etwas rotieren lassen. Der entstandene Wert wird dann wieder im EEPROM abgelegt:
___rseed=wEeRndInit    'Initialisierungswert aus dem EEPROM holen und Zufallsgenerator damit initialisieren
Rotate ___rseed,Left,7 'Die Bits des Initialisierungswerts rotieren lassen...
Incr ___rseed          'und den Wert um 1 erhhen. Damit sollte ein recht zuflliger Eindruck entstehen.
wEeRndInit=___rseed    'Und nun den neuen Wert im EEPROM ablegen.
'
'Morseton mit Timer1 erzeugen:
'Statt den Ton zu Fu durch Ein- und Ausschalten von Pin PB4 zu erzeugen, konfigurieren
'wir jetzt Timer1 so, da er dies selbstndig und unabhngig vom laufenden Programm fr
'uns tut. Sobald wir die entsprechenden 4 Timer1-Register beschickt haben, beginnt die
'Tonausgabe:
TCCR1=Bits(CS11,CS10,CTC1) 'Timer1: Vorteilung 1MHz/8, Timer-Reset bei Compare-Match
GTCCR=Bits(COM1B0)         'Timer1: Toggle Pin OC1B/PB4, wenn Register TCNT1 Wert von Compare-Register OCR1B hat
OCR1B=TONE_FREQU           'Timer1: Ausprobierter Vergleichswert. Wenn erreicht, dann Aktion an Pin OC1B/PB4
OCR1C=TONE_FREQU           'Timer1: Timer-Maximalwert wie Vergleichswert setzen
'-------------------------------------------------------------------------------
'Hauptprogramm:
WaitMs 500                             'Nicht zu schnell starten, damit die Hand rechtzeitig vom Einschalter zum Bleistift kommt
Do
  For CharNo=1 To 5                    '5 Morsebuchstaben ausgeben
    CharIndex=Rnd(36)+1                'Pseudo-Zufallszahl von 1...36: Aktuellen Morsebuchstaben festlegen
    i=Table(CharIndex) And &B00000111  'Die Anzahl der Punkte/Striche ist in den unteren 3 Bits...
    i=i+2                              'Anzahl in hchste verwendete Bitnummer umwandeln
    For BitNo=3 To i                   'Bit 3...i enthlt die Punkte und Striche
      DDRB.4=1                         'Timer1-Pin OC1B/PB4 als Ausgang konfigurieren: Ton an
      If Table(CharIndex).BitNo=0 Then 'Bit nicht gesetzt: Es ist ein Punkt (dit)
        WaitMs LEN_DIT                 'Jetzt warten, Timer1 gibt selbstndig den Ton aus
      Else                             'Bit gesetzt: Es ist ein Strich (da)
        WaitMs LEN_DA                  'Jetzt warten, Timer1 gibt selbstndig den Ton aus
      End If
      DDRB.4=0                         'Timer1-Pin OC1B/PB4 als Eingang konfigurieren: Ton aus
      WaitMs LEN_DIT                   'Pause innerhalb des Morsebuchstabens
    Next BitNo
    WaitMs LEN_DA                      'Pause zwischen den Morsebuchataben
  Next CharNo
  WaitMs LEN_PAUSE                     'Pause zwischen den 5er-Gruppen
Loop
End
'------------------------------------------------------------------------------------------------------------------------------
'Die folgende Tabelle enthlt die wichtigsten Morsezeichen in der Reihenfolge A,B,C...Z,0,1,2...9 (zusammen 36 Bytes).
'Die Bits jedes "Code"-Bytes haben folgende Bedeutung:
'1.) Die unteren 3 Bits geben an, wieviele Punkte/Striche in den oberen 5 Bits verwendet werden, wobei die Werte 0, 6 und 7
'    unzulssig sind.
'2.) Die oberen 5 Bits sind der Borsebuchstabe selbst. Eine 0 ist ein Punkt (dit) und eine 1 ist ein Strich (da). Der
'    Beginn ist das unterste der 5 oberen Bits. Es werden nur so viele der 5 Bits verwendet, wie in den untersten 3 Bits
'    angegeben sind. Die restlichen Bits werden ignoriert.
'Hier sind einige Codes als Beispiel:
'18  dezimal ist 00010010 binr: 010 = 2 Punkte/Striche, 01    = dit da              ==> Buchstabe A
'12  dezimal ist 00001100 binr: 100 = 4 Punkte/Striche, 1000  = da dit dit dit      ==> Buchstabe B
'44  dezimal ist 10101100 binr: 100 = 4 Punkte/Striche, 1010  = da dit da dit       ==> Buchstabe C
'11  dezimal ist 00001011 binr: 011 = 3 Punkte/Striche, 100   = da dit dit          ==> Buchstabe D
'1   dezimal ist 00000001 binr: 001 = 1 Punkt/Strich,   0     = dit                 ==> Buchstabe E
'36  dezimal ist 00100100 binr: 100 = 4 Punkte/Striche, 0010  = dit dit da dit      ==> Buchstabe F
'116 dezimal ist 01110100 binr: 100 = 4 Punkte/Striche, 0111  = dit da da da        ==> Buchstabe J
'108 dezimal ist 01101100 binr: 100 = 4 Punkte/Striche, 1011  = da dit da da        ==> Buchstabe Y
'28  dezimal ist 00011100 binr: 100 = 4 Punkte/Striche, 1100  = da da dit dit       ==> Buchstabe Z
'253 dezimal ist 11111101 binr: 101 = 5 Punkte/Striche, 11111 = da da da da da      ==> Buchstabe 0
'245 dezimal ist 11110101 binr: 101 = 5 Punkte/Striche, 01111 = dit da da da da     ==> Buchstabe 1
'133 dezimal ist 10000101 binr: 101 = 5 Punkte/Striche, 00001 = dit dit dit dit da  ==> Buchstabe 4
'61  dezimal ist 00111101 binr: 101 = 5 Punkte/Striche, 11100 = da da da dit dit    ==> Buchstabe 8
'125 dezimal ist 01111101 binr: 101 = 5 Punkte/Striche, 01111 = dit dit dit dit da  ==> Buchstabe 9
'... Ihr knnt die nicht aufgefhrten Codes selbst hinzufgen als bung, da ihr es verstanden habet ;-)
'Diese Art Codierung drfte so ziemlich das effizienteste sein, was man in einem Mikrocontroller
'machen kann.
'Mit der gewhlten Codierung sind natrlich nur Morsezeichen mit 1 bis 5 Punkten/Strichen darstellbar. Aber damit
'sind ja immerhin alle Buchstaben einschlielich deutscher und anderer Sonderzeichen und die Zahlen erreicht.
'Ein Problem sind aber die 6stelligen Satzzeichen und die 8stellige Irrung. Hierfr kann man die "illegalen"
'Lngenangaben 0, 6 und 7 nutzen.
'So knnte man z.B. vereinbaren, da bei einer Lngenangabe von 6 erst ein dit gesendet wird und dann die Bits, die
'im oberen 5-Bit-Block codiert sind. Frs Fragezeichen "dit dit da da dit dit" codiert man also "dit da da dit dit"
'mit Lngenangabe 6. Entsprechend gibt eine Lngenangabe von 7 an, da erst ein da gesendet wird und dann die Bits,
'die im oberen 5-Bit-Block codiert sind. Frs Komma "da da dit dit da da" codiert man also "da dit dit da da" mit
'Lngenangabe 7. Diese Art der Erweiterung ist softwareseitig sehr einfach umsetzbar.
'Damit haben wir bis auf eine einzige Ausnahme alle Zeichen erfat, denn auer zusammengesetzten Zeichen gibt es
'keine Morsezeichen, die mehr als 6 Stellen haben. Was uns noch fehlt, ist die Irrung mit 8 mal dit. Hier knnten
'wir die Lngenangabe 0 verwenden, der wir die Bedeutung geben, alle 8 Bits des Codes als Stellen des Morsezeichens
'anzusehen und auszusenden. Bei der Irrung setzen wir also Code 0, was dann 8 mal dit ergibt. Auch das ist
'softwareseitig gut umsetzbar. Da sich hierbei als reine Mglichkeit ein Reihe von 8stelligen Morsezeichen ergeben,
'die man gar nicht bentigt, kann die Lngenangabe 0 natrlich auch genutzt werden, um beliebige Steuerkommandos
'zu codieren - etwa um ein Funkgert auf Senden zu schalten oder irgend etwas anderes zu tun.
Morsecodes:   'Die Codes legen wir im Flash ab. Sie werden mit dem Kommando Read in ein Feld eingelesen
Data 18,12,44,11,1,36,27,4,2,116,43,20,26,10,59,52,92,19,3,9,35,68,51,76,108,28,253,245,229,197,133,5,13,29,61,125