'Programm:   Demo fr den UKW-Fuchsjagd-Sender "Foxy" mit ATtiny45+MICRF102
'            Sendet einen frei whlbaren Text in Morsezeichen. Das Programm be-
'            bentigt trotz Hochsprache nur etwa 10% des 4 KB groen Programmspeichers.
'Sprache:    Bascom-AVR-Basic, siehe www.mcselec.com (freie Version hat fr ATtiny45 keine Einschrnkungen)
'Von:        Roland Walter, DL7UNO, www.rowalt.de
'Datum:      08.02.2007 (letzte nderung)
'Literatur:  Roland Walter: AVR-Mikrocontroller-Lehrbuch, siehe www.rowalt.de
'Kommentar:  Pin PB4 wird als Timer1-Ausgang OC1B verwendet, Timer1 erzeugt den
'            Ton selbstndig.
'-------------------------------------------------------------------------------
'Grundeinstelllungen:
$regfile = "ATtiny45.dat"   'Deklarationen fr den ATtiny45 einbinden
$crystal = 1000000          'Taktfrequenz: 1 MHz (Fabrik-Voreinstellung, Interner RC-Oszillator 8MHz mitb Vorteilung 1/8)
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'Einstellbare Werte je nach Geschmack:
Const LEN_DIT    = 80          'Lnge eines Punkts, daraus ergibt sich die Morsegeschwindigkeit
Const LEN_DA     = LEN_DIT*3   'Lnge eines Strichs (mu 3*Punkt sein)
Const LEN_PAUSE  = LEN_DIT*6   'Lnge der Pause zwischen den 5er-Gruppen (optimal: 6 Punkt-Lngen)
Const TONE_FREQU = 140         'Tonfrequenz, Wert 140 ergab sich beim Ausprobieren als recht brauchbar
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'Die folgenden Konstanten nehmen den binren Code fr das betreffende Morsezeichen
'auf. Konstanten bentigen brigens KEINEN Platz im Flash, sie sind lediglich eine
'Art "bersetzungsanweisung" fr den Compiler. Die Erluterung fr die Codierung
'befindet sich ganz unten im Listing.
'ACHTUNG: Konzentrationsbung, daher keine 100%ige Gewhr fr Fehlerfreiheit.
Const MORSE_A=18:Const MORSE_B=12:Const MORSE_C=44:Const MORSE_D=11:Const MORSE_E=1
Const MORSE_F=36:Const MORSE_G=27:Const MORSE_H=4:Const MORSE_I=2:Const MORSE_J=116
Const MORSE_K=43:Const MORSE_L=20:Const MORSE_M=26:Const MORSE_N=10:Const MORSE_O=59
Const MORSE_P=52:Const MORSE_Q=92:Const MORSE_R=19:Const MORSE_S=3:Const MORSE_T=9
Const MORSE_U=35:Const MORSE_V=68:Const MORSE_W=51:Const MORSE_X=76:Const MORSE_Y=108
Const MORSE_Z=28
Const MORSE_AE=84:Const MORSE_OE=68:Const MORSE_UE=100:Const MORSE_CH=4
Const MORSE_NULL=253:Const MORSE_1=245:Const MORSE_2=229:Const MORSE_3=197:Const MORSE_4=133
Const MORSE_5=5:Const MORSE_6=13:Const MORSE_7=29:Const MORSE_8=61:Const MORSE_9=125
Const MORSE_STROKE=141  '"-"
Const MORSE_SLASH =77  '"/"
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'Variablen dimensionieren:
Dim i As Byte                  'Fr temporren Gebrauch
Dim CharNo As Byte             'Aktueller Morsebuchstabe
Dim BitNo As Byte              'Zhlt die Code-Bits des aktuellen Morsebuchstabens hoch
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'Konstanten und Variablen zum Text, der gesendet werden soll:
Const LEN_TEXT= 6              'Lnge des Texts, der gesendet werden soll (z.B. "DL0ZZZ-1"=8)
Dim TxText(LEN_TEXT) As Byte   'Feld fr die codierten Morsebuchstaben
TxText(1)=MORSE_T              'Code frs Morsezeichen, Tabelle siehe ganz unten
TxText(2)=MORSE_E              'Code frs Morsezeichen, Tabelle siehe ganz unten
TxText(3)=MORSE_S              'Code frs Morsezeichen, Tabelle siehe ganz unten
TxText(4)=MORSE_T              'Code frs Morsezeichen, Tabelle siehe ganz unten
TxText(5)=MORSE_STROKE         'Code frs Morsezeichen, Tabelle siehe ganz unten
TxText(6)=MORSE_1              'Code frs Morsezeichen, Tabelle siehe ganz unten
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'Initialisierung des Programms:
'
ACSR.ACD = 0           'Analog-Komparator ausschalten, spart etwas Strom
'
DDRB  = &B00011000     'Pins PB3+PB4 als Ausgnge konfigurieren (1), Rest ist Eingang (0)
PORTB = &B00000111     'PullUp-Widerstnde fr die Pins PB0,PB1,PB2 aktivieren (Strfestigkeit)
'
PORTB.3 = 1            'Pin PB3 auf High (StandBy-Pin am MICRF102): Standby-Modus abschalten
'
'Morseton mit Timer1 erzeugen:
'Statt den Ton zu Fu durch Ein- und Ausschalten von Pin PB4 zu erzeugen, konfigurieren
'wir jetzt Timer1 so, da er dies selbstndig und unabhngig vom laufenden Programm fr
'uns tut. Sobald wir die entsprechenden 4 Timer1-Register beschickt haben, beginnt die
'Tonausgabe:
TCCR1=Bits(CS11,CS10,CTC1) 'Timer1: Vorteilung 1MHz/8, Timer-Reset bei Compare-Match
GTCCR=Bits(COM1B0)         'Timer1: Toggle Pin OC1B/PB4, wenn Register TCNT1 Wert von Compare-Register OCR1B hat
OCR1B=TONE_FREQU           'Timer1: Ausprobierter Vergleichswert. Wenn erreicht, dann Aktion an Pin OC1B/PB4
OCR1C=TONE_FREQU           'Timer1: Timer-Maximalwert wie Vergleichswert setzen
'-------------------------------------------------------------------------------
'Hauptprogramm:
Do
  For CharNo=1 To LEN_TEXT             'Alle Morsebuchstaben ausgeben
    i=TxText(CharNo) And &B00000111    'Die Anzahl der Punkte/Striche ist in den unteren 3 Bits...
    i=i+2                              'Anzahl in hchste verwendete Bitnummer umwandeln
    For BitNo=3 To i                   'Bit 3...i enthlt die Punkte und Striche
      DDRB.4=1                         'Timer1-Pin OC1B/PB4 als Ausgang konfigurieren: Ton an
      If TxText(CharNo).BitNo=0 Then   'Bit nicht gesetzt: Es ist ein Punkt (dit)
        WaitMs LEN_DIT                 'Jetzt warten, Timer1 gibt selbstndig den Ton aus
      Else                             'Bit gesetzt: Es ist ein Strich (da)
        WaitMs LEN_DA                  'Jetzt warten, Timer1 gibt selbstndig den Ton aus
      End If
      DDRB.4=0                         'Timer1-Pin OC1B/PB4 als Eingang konfigurieren: Ton aus
      WaitMs LEN_DIT                   'Pause innerhalb des Morsebuchstabens
    Next BitNo
    WaitMs LEN_DA                      'Pause zwischen den Morsebuchataben
  Next CharNo
  WaitMs 1000                     'Pause, wenn Text komplett gesendet
Loop
End
'------------------------------------------------------------------------------------------------------------------------------
'Codierung der Morsebuchstaben:
'Jeder Morsebuchstabe wird mit einem Byte codiert. Die Bits jedes "Code"-Bytes haben folgende Bedeutung:
'1.) Die unteren 3 Bits geben an, wieviele Punkte/Striche in den oberen 5 Bits verwendet werden, wobei
'    die Werte 0, 6 und 7 unzulssig sind.
'2.) Die oberen 5 Bits sind der Borsebuchstabe selbst. Eine 0 ist ein Punkt (dit) und eine 1 ist ein
'    Strich (da). Der Beginn ist das unterste der 5 oberen Bits. Es werden nur so viele der 5 Bits
'    verwendet, wie in den untersten 3 Bits angegeben sind. Die restlichen Bits werden ignoriert.
'Hier sind einige Codes als Beispiel:
'18  dezimal ist 00010010 binr: 010 = 2 Punkte/Striche, 01    = dit da              ==> Buchstabe A
'12  dezimal ist 00001100 binr: 100 = 4 Punkte/Striche, 1000  = da dit dit dit      ==> Buchstabe B
'44  dezimal ist 10101100 binr: 100 = 4 Punkte/Striche, 1010  = da dit da dit       ==> Buchstabe C
'11  dezimal ist 00001011 binr: 011 = 3 Punkte/Striche, 100   = da dit dit          ==> Buchstabe D
'1   dezimal ist 00000001 binr: 001 = 1 Punkt/Strich,   0     = dit                 ==> Buchstabe E
'36  dezimal ist 00100100 binr: 100 = 4 Punkte/Striche, 0010  = dit dit da dit      ==> Buchstabe F
'116 dezimal ist 01110100 binr: 100 = 4 Punkte/Striche, 0111  = dit da da da        ==> Buchstabe J
'108 dezimal ist 01101100 binr: 100 = 4 Punkte/Striche, 1011  = da dit da da        ==> Buchstabe Y
'28  dezimal ist 00011100 binr: 100 = 4 Punkte/Striche, 1100  = da da dit dit       ==> Buchstabe Z
'253 dezimal ist 11111101 binr: 101 = 5 Punkte/Striche, 11111 = da da da da da      ==> Buchstabe 0
'245 dezimal ist 11110101 binr: 101 = 5 Punkte/Striche, 01111 = dit da da da da     ==> Buchstabe 1
'133 dezimal ist 10000101 binr: 101 = 5 Punkte/Striche, 00001 = dit dit dit dit da  ==> Buchstabe 4
'61  dezimal ist 00111101 binr: 101 = 5 Punkte/Striche, 11100 = da da da dit dit    ==> Buchstabe 8
'125 dezimal ist 01111101 binr: 101 = 5 Punkte/Striche, 01111 = dit dit dit dit da  ==> Buchstabe 9
'84  dezimal ist 01010100 binr: 100 = 4 Punkte/Striche, 0101  = dit da dit da       ==> Buchstabe 
'68  dezimal ist 01000100 binr: 100 = 4 Punkte/Striche, 0001  = da da da dit        ==> Buchstabe 
'100 dezimal ist 01100100 binr: 100 = 4 Punkte/Striche, 0011  = dit dit da da       ==> Buchstabe 
'4   dezimal ist 00000100 binr: 100 = 4 Punkte/Striche, 0000  = da da da da         ==> Buchstabe CH
'141 dezimal ist 10001101 binr: 101 = 5 Punkte/Striche, 10001 = da dit dit dit da   ==> Satzzeichen "-"
'77  dezimal ist 01001101 binr: 101 = 5 Punkte/Striche, 10010 = da dit dit da dit   ==> Satzzeichen "/"
'
'... Ihr knnt die nicht aufgefhrten Codes selbst hinzufgen als bung, da ihr es verstanden habt ;-)
'Diese Art Codierung drfte so ziemlich das effizienteste sein, was man in einem Mikrocontroller
'machen kann.
'Mit der gewhlten Codierung sind natrlich nur Morsezeichen mit 1 bis 5 Punkten/Strichen darstellbar. Aber damit
'sind ja immerhin alle Buchstaben einschlielich deutscher und anderer Sonderzeichen und die Zahlen erreicht.
'Ein Problem sind aber die 6stelligen Satzzeichen und die 8stellige Irrung. Hierfr kann man die "illegalen"
'Lngenangaben 0, 6 und 7 nutzen.
'So knnte man z.B. vereinbaren, da bei einer Lngenangabe von 6 erst ein dit gesendet wird und dann die Bits, die
'im oberen 5-Bit-Block codiert sind. Frs Fragezeichen "dit dit da da dit dit" codiert man also "dit da da dit dit"
'mit Lngenangabe 6. Entsprechend gibt eine Lngenangabe von 7 an, da erst ein da gesendet wird und dann die Bits,
'die im oberen 5-Bit-Block codiert sind. Frs Komma "da da dit dit da da" codiert man also "da dit dit da da" mit
'Lngenangabe 7. Diese Art der Erweiterung ist softwareseitig sehr einfach umsetzbar.
'Damit haben wir bis auf eine einzige Ausnahme alle Zeichen erfat, denn auer zusammengesetzten Zeichen gibt es
'keine Morsezeichen, die mehr als 6 Stellen haben. Was uns noch fehlt, ist die Irrung mit 8 mal dit. Hier knnten
'wir die Lngenangabe 0 verwenden, der wir die Bedeutung geben, alle 8 Bits des Codes als Stellen des Morsezeichens
'anzusehen und auszusenden. Bei der Irrung setzen wir also Code 0, was dann 8 mal dit ergibt. Auch das ist
'softwareseitig gut umsetzbar. Da sich hierbei als reine Mglichkeit ein Reihe von 8stelligen Morsezeichen ergeben,
'die man gar nicht bentigt, kann die Lngenangabe 0 natrlich auch genutzt werden, um beliebige Steuerkommandos
'zu codieren - etwa um ein Funkgert auf Senden zu schalten oder irgend etwas anderes zu tun.
