'Programm:   Programm fr den UKW-Fuchsjagd-Sender "Foxy" mit ATtiny45+MICRF102
'            Vollwertiges Fuchsjagd-Programm mit 12+1 whlbaren Rufzeichen und zwei
'            verschiedenen Pausen-Modi: Keine Pause zwischen den rufzeichen oder Pausen mit
'            pseudozuflliger Lnge zwischen 30sec und 3min.
'
'            Peilwettkampf-Rufzeichen laut deutschem Amateurfunkgesetz: MO,MOE,MOI,MOS,MOH,MO5.
'            Weitere Peil-Rufzeichen, die zum Teil international gebruchlich sind oder waren, aber
'            nicht in der deutschen Amateurfunkordung erwhnt werden: MOT,MOM,MOO,MOA,MOU,MOV,MO4
'            Letztere sind nur im freien ISM-Bereich zulssig, aber alle sind auswhlbar.
'            Die Einstellung erfolgt ber Pin MISO (PB1 am AVR); er wird mit einem Taster
'            oder einem Stck Draht so lange nach nach Gnd (Masse) gezogen, bis das gewnschte
'            Rufzeichen zu hren ist. Dann Taste sofort loslassen. Die Einstellung wird
'            dauerhaft (auch ohne Batterie im EEPROM) gespeichert. Die Reihenfolge ist
'            MO,MOE,MOI,MOS,MOH,MO5,MOT,MOM,MOO,MOA,MOU,MOV,MO4 und dann wieder von vorn.
'            Auerdem sind ber Pin SCK=PB2 zwei Arten Pause einstellbar. Entweder wird
'            permanent das Rufzeichen OHNE PAUSE ausgesendet (Einstellung 1) oder als
'            Einstellung 2 werden die Rufzeichen je 5 mal ausgesendet und danach erfolgt eine
'            Sendepause mit pseudo-zuflliger Lnge zwischen 30 Sekunden und 3 Minuten. Dann
'            wird wieder 5 mal das Rufzeichen gesendet u.s.w.
'            Einstellung der Pausen-Art: ber Pin SCK (PB2 am AVR); er wird mit einem Taster
'            oder einem Stck Draht so lange nach nach Gnd (Masse) gezogen, bis entweder
'            ein langer Ton zu hren ist oder ein sirrender Ton. Ein langer Ton zeigt an, da
'            keine Pausen zwischen den Rufen gemacht werden und der sirrende Ton zeigt an, da
'            Pausen zuflliger Lnge gemacht werden.
'
'            Das Programm bentigt etwa ein Drittel des 4 KB groen Programmspeichers.
'Von:        Roland Walter, DL7UNO, www.rowalt.de
'Sprache:    Bascom-AVR-Basic, siehe www.mcselec.com (freie Version hat fr ATtiny45 keine Einschrnkungen)
'Datum:      22.02.2007 (letzte nderung)
'Literatur:  Roland Walter: AVR-Mikrocontroller-Lehrbuch, siehe www.rowalt.de
'Kommentar:  Pin PB4 wird als Timer1-Ausgang OC1B verwendet, Timer1 erzeugt den Ton selbstndig.
'            Stromsparmglichkeiten wurden nur am MICRF102 genutzt. AVR-seitig wird man kaum einen
'            der Stromspar-Modi des ATtiny45 (idle,PowerDown u.s.w.) nutzen knnen, weil auch in
'            den Sendepausen der Systemtakt bentigt wird oder zumindest ein unabhngig laufender
'            Timer gebraucht wrde, der den AVR aus einem Schlafzustand aufwecken knnte. Haben wir aber
'            nicht ;-) - Wer hier feilen will, kann aber den AVR-Takt auf 128KHz umstellen oder aber
'            den Standard-RC-OSzillator generell so weit wie mglich heruntersetzen, wobei man
'            (bser Trick) die Frequenz des normalen RC-OSzillators ber seinen Kalibrier-Register
'            OSCCAL so verbiegt, da er in den Pausen langsamer luft. Bei CMOS sopart das ja Strom...
'            Aber im normalen Fuchsjagd-Programm schien mir das unntig zu sein, zumal das Programm
'            unbedingt bersichtlich und nachvollziehbar bleiben sollte. Dieser Punkt hat mich
'            brigens am meisten Zeit gekostet.
'-------------------------------------------------------------------------------
'Grundeinstelllungen:
$regfile = "ATtiny45.dat"   'Deklarationen fr den ATtiny45 einbinden
$crystal = 1000000          'Taktfrequenz: 1 MHz (Fabrik-Voreinstellung, Interner RC-Oszillator 8MHz mit Vorteilung 1/8)
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'Einstellbare Werte je nach Geschmack:
Const LEN_DIT    = 80          'Lnge eines Punkts, daraus ergibt sich die Morsegeschwindigkeit
Const LEN_DA     = LEN_DIT*3   'Lnge eines Strichs (mu 3*Punkt sein)
Const LEN_PAUSE  = LEN_DIT*6   'Lnge der Pause zwischen den 5er-Gruppen (optimal: 6 Punkt-Lngen)
Const TONE_FREQU = 140         'Tonfrequenz, Wert 140 ergab sich beim Ausprobieren als recht brauchbar
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'Die folgenden Konstanten nehmen den binren Code fr das betreffende Morsezeichen
'auf. Konstanten bentigen brigens KEINEN Platz im Flash, sie sind lediglich eine
'Art "bersetzungsanweisung" fr den Compiler. Die Erluterung fr die Codierung
'befindet sich ganz unten im Listing.
'ACHTUNG: Konzentrationsbung, daher keine 100%ige Gewhr fr Fehlerfreiheit.
Const MORSE_A=18:Const MORSE_B=12:Const MORSE_C=44:Const MORSE_D=11:Const MORSE_E=1
Const MORSE_F=36:Const MORSE_G=27:Const MORSE_H=4:Const MORSE_I=2:Const MORSE_J=116
Const MORSE_K=43:Const MORSE_L=20:Const MORSE_M=26:Const MORSE_N=10:Const MORSE_O=59
Const MORSE_P=52:Const MORSE_Q=92:Const MORSE_R=19:Const MORSE_S=3:Const MORSE_T=9
Const MORSE_U=35:Const MORSE_V=68:Const MORSE_W=51:Const MORSE_X=76:Const MORSE_Y=108
Const MORSE_Z=28
Const MORSE_AE=84:Const MORSE_OE=68:Const MORSE_UE=100:Const MORSE_CH=4
Const MORSE_NULL=253:Const MORSE_1=245:Const MORSE_2=229:Const MORSE_3=197:Const MORSE_4=133
Const MORSE_5=5:Const MORSE_6=13:Const MORSE_7=29:Const MORSE_8=61:Const MORSE_9=125
Const MORSE_STROKE=141  '"-"
Const MORSE_SLASH =77  '"/"
'
'Die folgenden Konstanten definieren die Pausenarten, die der Anwender whlen kann:
Const PAUSE_NULL=0             'Rufzeichen ohne Pause aussenden
Const PAUSE_RAND=1             'Rufzeichen mit pseudozuflliger Pause dazwischen aussenden
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'Variablen dimensionieren:
Dim i As Byte                  'Fr temporren Gebrauch im Hauptprogramm
Dim CharNo As Byte             'Aktueller Morsebuchstabe
Dim BitNo As Byte              'Zhlt die Code-Bits des aktuellen Morsebuchstabens hoch
Dim LenCurText As Byte         'Lnge des aktuell zu sendenden Rufzeichens in Zeichen (MO=2, MOS=3)
Dim RandPauseLen As Byte       'Zhler fr aktuelle Pausenlnge, wenn zufllige Pausenlngen eingestellt
Dim RandCalls As Byte          'Zhler fr die Anzahl der zu sendenden Rufzeichen zwischen den Pausen, wenn zufllige Pausenlngen eingestellt
Dim eeCurCallSign As ERAM Byte 'Aktuell eingestelltes Rufzeichen (0=MO,1=MOE,2=MOI....)
Dim eeCurPauseID As ERAM Byte  'Aktuell eingestellte Pausenart (Konstante PAUSE_...)
Dim fInitDone As Bit           'Ist beim Start Null, ist 1, wenn Initialisierungsroutinen komplett durchlaufen
Dim wEeRndInit As ERAM Word    'Initialisierungswert fr den Pseudo-Zufallsgenerator im EEPROM
Dim ___rseed As Word           'Initialisierungs-Variable des Pseudo-Zufallsgenerators, siehe Bascom-Hilfe unter "Rnd()"
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'Konstanten und Variablen zum Text, der gesendet werden soll:
Dim TxText(3) As Byte          'Feld fr die codierten Morsebuchstaben
'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
'Initialisierung des Programms:
'
ACSR.ACD = 0           'Analog-Komparator ausschalten, spart etwas Strom
'
DDRB  = &B00011001     'Pins PB0+PB3+PB4 als Ausgnge konfigurieren (1), Rest ist Eingang (0)
PORTB = &B00000110     'PullUp-Widerstnde fr die Pins PB1+PB2 aktivieren (Strfestigkeit)
'
If eeCurPauseID=PAUSE_RAND Then 'Nur wenn pseudozufllige Pausen vom Anwender ausgewhlt wurden
  'Damit der Pseudo-Zufallsgenerator nicht jedesmal die selbe Zahlenreihe erzeugt, mssen wir ihn mit
  'einem vernderlichen Startwert fttern. Wir lsen das auf einfachste Weise, indem wir hochzhlen,
  'wie oft das Programm bereits gestartet wurde und die Bits etwas rotieren lassen. Der entstandene Wert
  'wird dann wieder im EEPROM abgelegt:
  ___rseed=wEeRndInit    'Initialisierungswert aus dem EEPROM holen und Zufallsgenerator damit initialisieren
  Rotate ___rseed,Left,7 'Die Bits des Initialisierungswerts rotieren lassen...
  Incr ___rseed          'und den Wert um 1 erhhen. Damit sollte ein recht zuflliger Eindruck entstehen.
  wEeRndInit=___rseed    'Und nun den neuen Wert im EEPROM ablegen.
End If
'
'Morseton mit Timer1 erzeugen:
'Statt den Ton zu Fu durch Ein- und Ausschalten von Pin PB4 zu erzeugen, konfigurieren
'wir jetzt Timer1 so, da er dies selbstndig und unabhngig vom laufenden Programm fr
'uns tut. Sobald wir die entsprechenden 4 Timer1-Register beschickt haben, beginnt die
'Tonausgabe:
TCCR1=Bits(CS11,CS10,CTC1) 'Timer1: Vorteilung 1MHz/8, Timer-Reset bei Compare-Match
GTCCR=Bits(COM1B0)         'Timer1: Toggle Pin OC1B/PB4, wenn Register TCNT1 Wert von Compare-Register OCR1B hat
OCR1B=TONE_FREQU           'Timer1: Ausprobierter Vergleichswert. Wenn erreicht, dann Aktion an Pin OC1B/PB4
OCR1C=TONE_FREQU           'Timer1: Timer-Maximalwert wie Vergleichswert setzen
'-------------------------------------------------------------------------------
'Hauptprogramm:
GoSub Button1Pressed       'Simuliert Tastendruck zur Initialisierung (Rufzeichen laden)
fInitDone=1                'Alle Initialisierungsroutinen komplett durchlaufen
'
Do                                     'Hauptschleife des Programms
  'Reaktion auf Tastendrcke (Draht von Pin PB1 oder PB2 nach Gnd gezogen)
  'Hinweis: Wir brauchen hier keine Tasten-Entprellung (mit Debounce o..), weil wir an diesen Punkt nur
  'jeweils einmal nach Absenden eines kompletten Rufzeichens gelangen. Das wirkt schon ganz alleine als
  'Entprellung.
  If PinB.1=0 Then GoSub Button1Pressed   'Wenn Pin PB1 (=MISO) Null Volt
  If PinB.2=0 Then GoSub Button2Pressed   'Wenn Pin PB2 (=SCK) Null Volt
  '
  'Senden der Tne:
  'Bitte beachten: Wir beenden den StandBy-Zustand des MICRF102 erst unmittelbar vor Beginn eines Tons und
  'schalten den MICRF102 sofort nach Ende eines Tons wieder in den StandBy-Zustand. Das spart zum einen eine
  'Menge Strom, weil Mark und Space bei ASK voller und verminderter Trger bedeuten, d.h. auch bei Space wird
  'ein Trger gesendet! Zum zweiten erlichtern wir aber das Peilen gewaltig, wenn zwischen den Tnen kein
  'Trger stehenbleibt. Dann kann beim Peilen auch das Rauschen zwischen den Tnen zur Beurteilung herangezogen
  'werden, was insbesondere bei Empfngern mit automatischer Empfindlichkeitsregelung sehr hilfreich ist.
  For CharNo=1 To LenCurText           'Alle Morsebuchstaben ausgeben
    i=TxText(CharNo) And &B00000111    'Die Anzahl der Punkte/Striche ist in den unteren 3 Bits...
    i=i+2                              'Anzahl in hchste verwendete Bitnummer umwandeln
    For BitNo=3 To i                   'Bit 3...i enthlt die Punkte und Striche
      PORTB.3=1                        'Pin PB3 High (StandBy am MICRF102 beenden): Hinweis: Dauert etwa 6 ms laut MICRF102-Datasheet
      DDRB.4=1                         'Timer1-Pin OC1B/PB4 als Ausgang konfigurieren: Ton an
      PORTB.0=1                        'Pin PB0/MOSI auf High: LED-Ausgang High
      If TxText(CharNo).BitNo=0 Then   'Bit nicht gesetzt: Es ist ein Punkt (dit)
        WaitMs LEN_DIT                 'Jetzt warten, Timer1 gibt selbstndig den Ton aus
      Else                             'Bit gesetzt: Es ist ein Strich (da)
        WaitMs LEN_DA                  'Jetzt warten, Timer1 gibt selbstndig den Ton aus
      End If
      DDRB.4=0                         'Timer1-Pin OC1B/PB4 als Eingang konfigurieren: Ton aus
      PORTB.0=0                        'Pin PB0/MOSI auf High: LED-Ausgang Low
      PORTB.3=0                        'Pin PB3 auf Low (StandBy-Pin am MICRF102): Standby-Modus aktivieren
      WaitMs LEN_DIT                   'Pause innerhalb des Morsebuchstabens
    Next BitNo
    WaitMs LEN_DA                      'Pause zwischen den Morsebuchataben
  Next CharNo
  '
  'Pause laut Anwender-Auswahl:
  Select Case eeCurPauseID
  Case PAUSE_NULL              'PAUSE_NULL=0: Rufzeichen ohne Pause aussenden
    WaitMs 1000                'Stets 1 Sekunde Pause, wenn Text komplett gesendet
  Case Else                    'PAUSE_RAND=1: Rufzeichen mit pseudozuflliger Pause dazwischen aussenden
    WaitMs 1000                'Stets 1 Sekunde Pause, wenn Text komplett gesendet
    Incr RandCalls             'Zwischen den Pausen wird jeweils 5 mal das Rufzeichen gesendet
    If RandCalls>4 Then        'Das Rufzeichen wurde 5 mal gesendet: Pause machen
      RandCalls=0              'Zhler zurksetzen
      i=Rnd(150)+30            'Pseudo-Zufallszahl von 30...150+30=180 fr 30sec...3min Pause
      For RandPauseLen=0 To i  'Runden mit je einer Sekunde Pause, in denen (wichtig!) der Tasten-Status geprft wird
        WaitMs 1000
        If PinB.1=0 Then Exit For   'Bei Button-Druck Schleife sofort verlassen, Auswertung siehe oben
        If PinB.2=0 Then Exit For   'Bei Button-Druck Schleife sofort verlassen, Auswertung siehe oben
      Next RandPauseLen
    End If
  End Select
Loop
'------------------------------------------------------------------------------------------------------------------------------
Button1Pressed:            'Button 1 (Pin PB1 nach Gnd) wurde gedrckt
  'Das Rufzeichen soll gendert werden. Die ID des aktuellen Rufzeichens wird stets in der
  'EEPROM-Variablen eeCurCallSign gespeichert. Abhngig vom Wert dieser Variablen wird
  'das Feld TxText() verndert, welches die Morsecodes frs Senden speichert.
  '
  RandCalls=0                 'Ruf-Zhler zurcksetzen, falls gerade Zufalls-Pausen aktiviert sind
  '
  TxText(1)=MORSE_M           'Erster Buchstabe des Rufzeichens   (bei allen Fuchs-Rufzeichen gleich)
  TxText(2)=MORSE_O           'Zweiter Buchstabe des Rufzeichens  (bei allen Fuchs-Rufzeichen gleich)
  LenCurText=3                'Lnge des aktuellen Rufzeichens in Zeichen (bleibt so auer bei MO)
  '
  i=eeCurCallSign             'Aktuelle Rufzeichen-ID vom EEPROM in Temp-Variable laden
  If fInitDone=1 Then Incr i  'Nchstes Rufzeichen, falls nicht Initialisierung des Programms
  If i>12 Then i=0            'Umlauf, wieder Rufzeichen MO (auch bei Fabrikzustand=255)
  '
  'Rufzeichen-Vorrat: MO,MOE,MOI,MOS,MOH,MO5,MOT,MOM,MOO,MOA,MOU,MOV,MO4
  Select Case i
  Case 0:  LenCurText=2       'MO: Sonderfall, es gibt keinen dritten Buchstaben
  Case 1:  TxText(3)=MORSE_E
  Case 2:  TxText(3)=MORSE_I
  Case 3:  TxText(3)=MORSE_S
  Case 4:  TxText(3)=MORSE_H
  Case 5:  TxText(3)=MORSE_5
  Case 6:  TxText(3)=MORSE_T
  Case 7:  TxText(3)=MORSE_M
  Case 8:  TxText(3)=MORSE_O
  Case 9:  TxText(3)=MORSE_A
  Case 10: TxText(3)=MORSE_U
  Case 11: TxText(3)=MORSE_V
  Case Else: TxText(3)=MORSE_4
  End Select
  '
  eeCurCallSign=i             'Aktuelle Rufzeichen-ID im EEPROM speichern
Return
'------------------------------------------------------------------------------------------------------------------------------
Button2Pressed:             'Button 2 (Pin PB2 nach Gnd) wurde gedrckt
  'Die Pausenart zwischen den Rufzeichen-Aussendeungen soll gendert werden. Die Art der Pause wird
  'in der EEPROM-Variablen eeCurPauseID gespeichert (Konstanten PAUSE_...).  Abhngig vom Wert
  'dieser Variablen wird die Variable CurPause gsetzt.
  '
  i=eeCurPauseID             'Aktuelle Pausen-ID vom EEPROM in Temp-Variable laden
  Incr i                     'Nchste Pausen-ID
  If i>1 Then i=0            'Umlauf, wieder PAUSE_NULL (auch bei Fabrikzustand=255)
  '
  'Quittungston senden: Langer Ton fr pausenfreies Senden, sirrender Ton fr Zufalls-Pausen
  '
  PORTB.3=1                  'Pin PB3 auf High (StandBy-Pin am MICRF102): Standby-Modus abschalten
  Select Case i
  Case PAUSE_NULL            'PAUSE_NULL=0: Rufzeichen ohne Pause aussenden
    'Langer Ton als Quittung:
    DDRB.4=1                 'Timer1-Pin OC1B/PB4 als Ausgang konfigurieren: Ton an
    WaitMs 3000              'Jetzt warten, Timer1 gibt selbstndig den Ton aus
    DDRB.4=0                 'Timer1-Pin OC1B/PB4 als Eingang konfigurieren: Ton aus
  Case Else                  'PAUSE_RAND=1: Rufzeichen mit pseudozuflliger Pause dazwischen aussenden
    RandCalls=5              'Ruf-Zhler so setzen, da 1x das Rufzeichen gesendet wird und dann sofort Pause (vermeidet Verwirrungen)
    'Sirrender Ton als Quittung:
    For i=0 To 50
      DDRB.4=1               'Timer1-Pin OC1B/PB4 als Ausgang konfigurieren: Ton an
      WaitMs 20              '20 Millisekunden warten, Timer1 gibt selbstndig den Ton aus
      DDRB.4=0               'Timer1-Pin OC1B/PB4 als Eingang konfigurieren: Ton aus
      WaitMs 20              '20 Millisekunden warten (Pause)
    Next i
  End Select
  PORTB.3=0                  'Pin PB3 auf Low (StandBy-Pin am MICRF102): Standby-Modus aktivieren
  '
  eeCurPauseID=i             'Aktuelle Pausen-ID im EEPROM speichern
  '
  'Die gewhlte Pause wird in der Hauptschleife generiert
Return
'------------------------------------------------------------------------------------------------------------------------------
End
'------------------------------------------------------------------------------------------------------------------------------
'Codierung der Morsebuchstaben (hoffentlich alle fehlerfrei):
'Jeder Morsebuchstabe wird mit einem Byte codiert. Die Bits jedes "Code"-Bytes haben folgende Bedeutung:
'1.) Die unteren 3 Bits geben an, wieviele Punkte/Striche in den oberen 5 Bits verwendet werden, wobei
'    die Werte 0, 6 und 7 unzulssig sind.
'2.) Die oberen 5 Bits sind der Borsebuchstabe selbst. Eine 0 ist ein Punkt (dit) und eine 1 ist ein
'    Strich (da). Der Beginn ist das unterste der 5 oberen Bits. Es werden nur so viele der 5 Bits
'    verwendet, wie in den untersten 3 Bits angegeben sind. Die restlichen Bits werden ignoriert.
'Hier sind einige Codes als Beispiel:
'18  dezimal ist 00010010 binr: 010 = 2 Punkte/Striche, 01    = dit da              ==> Buchstabe A
'12  dezimal ist 00001100 binr: 100 = 4 Punkte/Striche, 1000  = da dit dit dit      ==> Buchstabe B
'44  dezimal ist 10101100 binr: 100 = 4 Punkte/Striche, 1010  = da dit da dit       ==> Buchstabe C
'11  dezimal ist 00001011 binr: 011 = 3 Punkte/Striche, 100   = da dit dit          ==> Buchstabe D
'1   dezimal ist 00000001 binr: 001 = 1 Punkt/Strich,   0     = dit                 ==> Buchstabe E
'36  dezimal ist 00100100 binr: 100 = 4 Punkte/Striche, 0010  = dit dit da dit      ==> Buchstabe F
'116 dezimal ist 01110100 binr: 100 = 4 Punkte/Striche, 0111  = dit da da da        ==> Buchstabe J
'108 dezimal ist 01101100 binr: 100 = 4 Punkte/Striche, 1011  = da dit da da        ==> Buchstabe Y
'28  dezimal ist 00011100 binr: 100 = 4 Punkte/Striche, 1100  = da da dit dit       ==> Buchstabe Z
'253 dezimal ist 11111101 binr: 101 = 5 Punkte/Striche, 11111 = da da da da da      ==> Buchstabe 0
'245 dezimal ist 11110101 binr: 101 = 5 Punkte/Striche, 01111 = dit da da da da     ==> Buchstabe 1
'133 dezimal ist 10000101 binr: 101 = 5 Punkte/Striche, 00001 = dit dit dit dit da  ==> Buchstabe 4
'61  dezimal ist 00111101 binr: 101 = 5 Punkte/Striche, 11100 = da da da dit dit    ==> Buchstabe 8
'125 dezimal ist 01111101 binr: 101 = 5 Punkte/Striche, 01111 = dit dit dit dit da  ==> Buchstabe 9
'84  dezimal ist 01010100 binr: 100 = 4 Punkte/Striche, 0101  = dit da dit da       ==> Buchstabe 
'68  dezimal ist 01000100 binr: 100 = 4 Punkte/Striche, 0001  = da da da dit        ==> Buchstabe 
'100 dezimal ist 01100100 binr: 100 = 4 Punkte/Striche, 0011  = dit dit da da       ==> Buchstabe 
'4   dezimal ist 00000100 binr: 100 = 4 Punkte/Striche, 0000  = da da da da         ==> Buchstabe CH
'141 dezimal ist 10001101 binr: 101 = 5 Punkte/Striche, 10001 = da dit dit dit da   ==> Satzzeichen "-"
'77  dezimal ist 01001101 binr: 101 = 5 Punkte/Striche, 10010 = da dit dit da dit   ==> Satzzeichen "/"
'255 dezimal ist 11111111 binr: 111 = 7 Punkte/Striche, 11111 = illegal,Sondercode  ==> Pause mit Lnge Dit
'
'
'... Ihr knnt die nicht aufgefhrten Codes selbst hinzufgen als bung, da ihr es verstanden habt ;-)
'Diese Art Codierung drfte so ziemlich das effizienteste sein, was man in einem Mikrocontroller
'machen kann.
'Mit der gewhlten Codierung sind natrlich nur Morsezeichen mit 1 bis 5 Punkten/Strichen darstellbar. Aber damit
'sind ja immerhin alle Buchstaben einschlielich deutscher und anderer Sonderzeichen und die Zahlen erreicht.
'Ein Problem sind aber die 6stelligen Satzzeichen und die 8stellige Irrung. Hierfr kann man die "illegalen"
'Lngenangaben 0, 6 und 7 nutzen.
'So knnte man z.B. vereinbaren, da bei einer Lngenangabe von 6 erst ein dit gesendet wird und dann die Bits, die
'im oberen 5-Bit-Block codiert sind. Frs Fragezeichen "dit dit da da dit dit" codiert man also "dit da da dit dit"
'mit Lngenangabe 6. Entsprechend gibt eine Lngenangabe von 7 an, da erst ein da gesendet wird und dann die Bits,
'die im oberen 5-Bit-Block codiert sind. Frs Komma "da da dit dit da da" codiert man also "da dit dit da da" mit
'Lngenangabe 7. Diese Art der Erweiterung ist softwareseitig sehr einfach umsetzbar.
'Damit haben wir bis auf eine einzige Ausnahme alle Zeichen erfat, denn auer zusammengesetzten Zeichen gibt es
'keine Morsezeichen, die mehr als 6 Stellen haben. Was uns noch fehlt, ist die Irrung mit 8 mal dit. Hier knnten
'wir die Lngenangabe 0 verwenden, der wir die Bedeutung geben, alle 8 Bits des Codes als Stellen des Morsezeichens
'anzusehen und auszusenden. Bei der Irrung setzen wir also Code 0, was dann 8 mal dit ergibt. Auch das ist
'softwareseitig gut umsetzbar. Da sich hierbei als reine Mglichkeit ein Reihe von 8stelligen Morsezeichen ergeben,
'die man gar nicht bentigt, kann die Lngenangabe 0 natrlich auch genutzt werden, um beliebige Steuerkommandos
'zu codieren - etwa um ein Funkgert auf Senden zu schalten oder irgend etwas anderes zu tun.